@@ -99,7 +99,7 @@ if louis and not tablesdir:
 #
 _monitor = None
 
-# brlapi keys which are not allowed to interupt speech:
+# brlapi keys which are not allowed to interrupt speech:
 #
 dontInteruptSpeechKeys = []
 if _brlAPIAvailable:
@@ -354,6 +354,10 @@ class Region:
         else:
             mode = louis.compbrlAtCursor
 
+        msg = "BRAILLE: '%s', cursorPos: %i, mode: %s (%s)" % \
+              (line, mode, cursorOffset, self.contractionTable)
+        debug.println(debug.LEVEL_INFO, msg, True)
+
         contracted, inPos, outPos, cursorPos = \
             louis.translate([self.contractionTable],
                             line,
