@@ -54,7 +54,7 @@ class Utilities(script_utilities.Utilities):
 
         # Present GtkLabel children inside a GtkListBox row.
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
-            labels = self.unrelatedLabels(obj, onlyShowing=False)
+            labels = self.unrelatedLabels(obj)
             displayedText = " ".join(map(self.displayedText, labels))
 
         self._script.generatorCache[self.DISPLAYED_TEXT][obj] = displayedText
@@ -183,7 +183,7 @@ class Utilities(script_utilities.Utilities):
         return rv
 
     def rgbFromString(self, attributeValue):
-        regex = re.compile("rgb|[^\w,]", re.IGNORECASE)
+        regex = re.compile(r"rgb|[^\w,]", re.IGNORECASE)
         string = re.sub(regex, "", attributeValue)
         red, green, blue = string.split(",")
 
