@@ -485,6 +485,15 @@ class Script(default.Script):
             if event.source.getRole() == pyatspi.ROLE_LINK:
                 return False
 
+        if event.type.startswith('object:children-changed'):
+            try:
+                role = event.any_data.getRole()
+            except:
+                pass
+            else:
+                if role == pyatspi.ROLE_DIALOG:
+                    return False
+
         return super().skipObjectEvent(event)
 
     def consumesKeyboardEvent(self, keyboardEvent):
@@ -610,11 +619,14 @@ class Script(default.Script):
         if not (text and text.getNSelections()):
             return
 
-        context = self.utilities.getCaretContext(documentFrame=None)
+        document = self.utilities.getDocumentForObject(obj)
+        if not document:
+            return
 
+        context = self.utilities.getCaretContext(documentFrame=document)
         start, end = text.getSelection(0)
         offset = max(offset, start)
-        self.utilities.setCaretContext(obj, offset, documentFrame=None)
+        self.utilities.setCaretContext(obj, offset, documentFrame=document)
         if end - start < _settingsManager.getSetting('findResultsMinimumLength'):
             return
 
@@ -624,7 +636,7 @@ class Script(default.Script):
 
         if self._madeFindAnnouncement \
            and verbosity == settings.FIND_SPEAK_IF_LINE_CHANGED \
-           and not self.utilities.contextsAreOnSameLine(context, (obj, offset)):
+           and self.utilities.contextsAreOnSameLine(context, (obj, offset)):
             return
 
         contents = self.utilities.getLineContentsAtOffset(obj, offset)
@@ -833,6 +845,16 @@ class Script(default.Script):
         self.displayContents(contents)
         self.speakContents(contents, priorObj=priorObj)
  
+    def updateBrailleForNewCaretPosition(self, obj):
+        """Try to reposition the cursor without having to do a full update."""
+
+        text = self.utilities.queryNonEmptyText(obj)
+        if text and self.EMBEDDED_OBJECT_CHARACTER in text.getText(0, -1):
+            self.updateBraille(obj)
+            return
+
+        super().updateBrailleForNewCaretPosition(obj)
+
     def updateBraille(self, obj, **args):
         """Updates the braille display to show the given object."""
 
@@ -847,9 +869,12 @@ class Script(default.Script):
             super().updateBraille(obj, **args)
             return
 
+        isContentEditable = self.utilities.isContentEditableWithEmbeddedObjects(obj)
+
         if not self._lastCommandWasCaretNav \
            and not self._lastCommandWasStructNav \
-           and not self.utilities.isContentEditableWithEmbeddedObjects(obj) \
+           and not isContentEditable \
+           and not self.utilities.isPlainText() \
            and not self.utilities.lastInputEventWasCaretNavWithSelection():
             msg = "WEB: updating braille for unhandled navigation type %s" % obj
             debug.println(debug.LEVEL_INFO, msg, True)
@@ -857,6 +882,10 @@ class Script(default.Script):
             return
 
         obj, offset = self.utilities.getCaretContext(documentFrame=None)
+        if offset > 0 and isContentEditable:
+            text = self.utilities.queryNonEmptyText(obj)
+            offset = min(offset, text.characterCount)
+
         contents = self.utilities.getLineContentsAtOffset(obj, offset)
         self.displayContents(contents)
 
@@ -1091,11 +1120,6 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return False
 
-        if oldFocus and self.utilities.isZombie(oldFocus):
-            msg = "WEB: Old focus is Zombie: %s. Clearing oldFocus." % oldFocus
-            debug.println(debug.LEVEL_INFO, msg, True)
-            oldFocus = None
-
         caretOffset = 0
         if not oldFocus or self.utilities.inFindToolbar(oldFocus):
             contextObj, contextOffset = self.utilities.getCaretContext()
@@ -1114,7 +1138,12 @@ class Script(default.Script):
         self.utilities.setCaretContext(newFocus, caretOffset)
         self.updateBraille(newFocus)
 
-        if self.utilities.isAnchor(newFocus):
+        if self.utilities.isContentEditableWithEmbeddedObjects(newFocus):
+            msg = "WEB: New focus %s content editable. Generating line contents." % newFocus
+            debug.println(debug.LEVEL_INFO, msg, True)
+            contents = self.utilities.getLineContentsAtOffset(newFocus, caretOffset)
+            utterances = self.speechGenerator.generateContents(contents)
+        elif self.utilities.isAnchor(newFocus):
             msg = "WEB: New focus %s is anchor. Generating line contents." % newFocus
             debug.println(debug.LEVEL_INFO, msg, True)
             contents = self.utilities.getLineContentsAtOffset(newFocus, 0)
@@ -1171,6 +1200,16 @@ class Script(default.Script):
 
         return False
 
+    def onActiveDescendantChanged(self, event):
+        """Callback for object:active-descendant-changed accessibility events."""
+
+        if not self.utilities.inDocumentContent(event.source):
+            msg = "WEB: Event source is not in document content"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return False
+
+        return True
+
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
@@ -1327,14 +1366,10 @@ class Script(default.Script):
             return True
 
         if self.utilities.inFindToolbar():
-            if not self._madeFindAnnouncement:
-                msg = "WEB: Event handled: Presenting find results"
-                debug.println(debug.LEVEL_INFO, msg, True)
-                self.presentFindResults(event.source, event.detail1)
-            else:
-                self.utilities.setCaretContext(event.source, event.detail1)
-                msg = "WEB: Event handled: Setting context to source and offset"
-                debug.println(debug.LEVEL_INFO, msg, True)
+            msg = "WEB: Event handled: Presenting find results"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            self.presentFindResults(event.source, event.detail1)
+            self._saveFocusedObjectInfo(orca_state.locusOfFocus)
             return True
 
         if self.utilities.eventIsAutocompleteNoise(event):
@@ -1367,14 +1402,15 @@ class Script(default.Script):
             msg = "WEB: Event handled: Caret moved due to scrolling"
             debug.println(debug.LEVEL_INFO, msg, True)
             self.utilities.setCaretContext(obj, offset)
-            orca.setLocusOfFocus(event, obj)
+            orca.setLocusOfFocus(event, obj, force=self.utilities.isPlainText())
             return True
 
         if self.utilities.isContentEditableWithEmbeddedObjects(event.source):
             msg = "WEB: In content editable with embedded objects"
             debug.println(debug.LEVEL_INFO, msg, True)
             self.utilities.setCaretContext(obj, offset)
-            orca.setLocusOfFocus(event, event.source, True)
+            notify = not self.utilities.lastInputEventWasCharNav()
+            orca.setLocusOfFocus(event, event.source, notify)
             return False
 
         text = self.utilities.queryNonEmptyText(event.source)
@@ -1640,9 +1676,10 @@ class Script(default.Script):
 
             obj, offset = self.utilities.searchForCaretContext(event.source)
             if obj:
+                notify = self.utilities.inFindToolbar(orca_state.locusOfFocus)
                 msg = "WEB: Updating focus and context to %s, %i" % (obj, offset)
                 debug.println(debug.LEVEL_INFO, msg, True)
-                orca.setLocusOfFocus(event, obj, False)
+                orca.setLocusOfFocus(event, obj, notify)
                 self.utilities.setCaretContext(obj, offset)
             else:
                 msg = "WEB: Search for caret context failed"
@@ -1831,6 +1868,11 @@ class Script(default.Script):
 
         obj, offset = self.utilities.getCaretContext(getZombieReplicant=False)
         if self.utilities.isZombie(obj):
+            if self.utilities.isLink(obj):
+                msg = "WEB: Focused link deleted. Taking no further action."
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return True
+
             obj, offset = self.utilities.getCaretContext(getZombieReplicant=True)
             if obj:
                 orca.setLocusOfFocus(event, obj, notifyScript=False)
@@ -1849,8 +1891,8 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             self.structuralNavigation.clearCache(document)
 
-        state = event.source.getState()
-        if not state.contains(pyatspi.STATE_EDITABLE):
+        if not event.source.getState().contains(pyatspi.STATE_EDITABLE) \
+           and not self.utilities.isContentEditableWithEmbeddedObjects(event.source):
             if self._inMouseOverObject \
                and self.utilities.isZombie(self._lastMouseOverObject):
                 msg = "WEB: Restoring pre-mouseover context"
@@ -1948,13 +1990,6 @@ class Script(default.Script):
             debug.println(debug.LEVEL_INFO, msg, True)
             return False
 
-        if self.utilities.inFindToolbar():
-            msg = "WEB: Event handled: Presenting find results"
-            debug.println(debug.LEVEL_INFO, msg, True)
-            self.presentFindResults(event.source, -1)
-            self._saveFocusedObjectInfo(orca_state.locusOfFocus)
-            return True
-
         if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
             msg = "WEB: Event ignored: locusOfFocus (%s) is not in document content" \
                   % orca_state.locusOfFocus
